@@ -21,6 +21,7 @@ SAMPLING_DEADZONE = 2
 class Calibration(object):
     manual_calibration_trigger = True
     linear_calibration_start_flag = False
+    stretch_calibration_flag = False
 
     linear_offset_template = {"INTERCEPT": 0.00, "SLOPE": 0.00}
     linear_offset = [linear_offset_template, linear_offset_template, linear_offset_template, linear_offset_template]
@@ -30,6 +31,7 @@ class Calibration(object):
     temp_manual_offset_arr = [0.00, 0.00, 0.00]
     manual_offset_correct_flag = [False, False, False]
 
+
     def __init__(self, uarm=None, log_function=None):
         if uarm is not None:
             self.uarm = uarm
@@ -81,7 +83,7 @@ class Calibration(object):
             self.write_completed_flag(CALIBRATION_FLAG, True)
             self.uf_print("Calibration DONE!!")
 
-    def linear_calibration_section(self, callback):
+    def linear_calibration_section(self, callback=None):
         self.linear_calibration_start_flag = True
         self.uf_print("1.0. Clearing Linear Completed Flag in EEPROM.")
         self.write_completed_flag(CALIBRATION_LINEAR_FLAG, False)
@@ -111,10 +113,10 @@ class Calibration(object):
             self.linear_calibration_start_flag = False
         else:
             self.uf_print("Error - 1. Linear Offset not equal to EEPROM, Please retry.")
-            self.uf_print("Error - 1.servo_offset: ", self.temp_manual_offset_arr)
-            self.uf_print("Error - 1.read_linear_offset(): ", self.read_linear_offset())
+            # self.uf_print("Error - 1. manual_offset: " + self.temp_manual_offset_arr)
+            self.uf_print("Error - 1. linear_offset(): " + self.read_linear_offset())
             self.linear_calibration_start_flag = False
-        self.uarm.detachAll()
+        self.uarm.detach_all_servos()
 
     def calibrate_linear_servo_offset(self, number):
         moveTimes = 5
@@ -129,38 +131,38 @@ class Calibration(object):
             angle = angle_step * moveTimes + ServoRangeIni
             if number == SERVO_ROT_NUM:
                 analog_read_pin = SERVO_ROT_ANALOG_PIN
-                self.uarm.writeServoAngle(SERVO_ROT_NUM, angle, 0)
-                self.uarm.writeLeftRightServoAngle(60, 30, 0)
+                self.uarm.write_servo_angle(SERVO_ROT_NUM, angle, 0)
+                self.uarm.write_left_right_servo_angle(60, 30, 0)
 
             if number == SERVO_LEFT_NUM:
                 analog_read_pin = SERVO_LEFT_ANALOG_PIN
-                self.uarm.writeServoAngle(SERVO_ROT_NUM, 90, 0)
-                self.uarm.writeLeftRightServoAngle(angle, 30, 0)
+                self.uarm.write_servo_angle(SERVO_ROT_NUM, 90, 0)
+                self.uarm.write_left_right_servo_angle(angle, 30, 0)
 
             if number == SERVO_RIGHT_NUM:
                 analog_read_pin = SERVO_RIGHT_ANALOG_PIN
-                self.uarm.writeServoAngle(SERVO_ROT_NUM, 90, 0)
-                self.uarm.writeLeftRightServoAngle(30, angle, 0)
+                self.uarm.write_servo_angle(SERVO_ROT_NUM, 90, 0)
+                self.uarm.write_left_right_servo_angle(30, angle, 0)
 
             if number == SERVO_HAND_ROT_NUM:
                 analog_read_pin = SERVO_HAND_ROT_ANALOG_PIN
-                self.uarm.writeServoAngle(SERVO_ROT_NUM, 90, 0)
-                self.uarm.writeLeftRightServoAngle(30, 60, 0)
-                self.uarm.writeServoAngle(SERVO_HAND_ROT_NUM, angle, 0)
+                self.uarm.write_servo_angle(SERVO_ROT_NUM, 90, 0)
+                self.uarm.write_left_right_servo_angle(30, 60, 0)
+                self.uarm.write_servo_angle(SERVO_HAND_ROT_NUM, angle, 0)
 
             if angle_step == 0:
-                time.sleep(0.5)
+                time.sleep(0.1)
             else:
                 time.sleep(0.1)
 
             for i in range(2):
-                servo_analog_read = self.uarm.readAnalog(analog_read_pin)
+                servo_analog_read = self.uarm.read_analog(analog_read_pin)
                 time.sleep(0.1)
 
             angles.append(angle)
             analogs.append(servo_analog_read)
             angle_step += 1
-            time.sleep(0.2)
+            time.sleep(0.1)
 
         new_ab = basic_linear_regression(analogs, angles)
         linear_offset_template = copy.deepcopy(self.linear_offset_template)
@@ -173,16 +175,16 @@ class Calibration(object):
         # self.ab_values_b.append(new_ab[1])
         # self.complete_display(number,'linear')
 
-    def manual_calibration_section(self, callback):
+    def manual_calibration_section(self, callback=None):
         self.uf_print("2.0. Clearing Servo Completed Flag in EEPROM.")
         self.write_completed_flag(CALIBRATION_SERVO_FLAG, False)
         self.uf_print("2. Start Calibrate Servo Offset")
         self.manual_operation_trigger = True
-        self.uarm.writeServoAngle(SERVO_ROT_NUM, 45, 0)
+        self.uarm.write_servo_angle(SERVO_ROT_NUM, 45, 0)
         time.sleep(1)
-        self.uarm.writeLeftRightServoAngle(130, 20, 0)
+        self.uarm.write_left_right_servo_angle(130, 20, 0)
         time.sleep(1)
-        self.uarm.detachAll()
+        self.uarm.detach_all_servos()
         time.sleep(0.3)
 
         time_counts = 0
@@ -191,11 +193,12 @@ class Calibration(object):
         servo_2_offset = 0
         servo_3_offset = 0
         self.uf_print("Please move uArm in right position")
+        self.uarm.alert(3,100,100)
         while self.manual_operation_trigger:
 
-            servo_1_offset = self.uarm.readServoAngle(SERVO_ROT_NUM, 0) - 45
-            servo_2_offset = self.uarm.readServoAngle(SERVO_LEFT_NUM, 0) - 130
-            servo_3_offset = self.uarm.readServoAngle(SERVO_RIGHT_NUM, 0) - 20
+            servo_1_offset = self.uarm.read_servo_angle(SERVO_ROT_NUM, 0) - 45
+            servo_2_offset = self.uarm.read_servo_angle(SERVO_LEFT_NUM, 0) - 130
+            servo_3_offset = self.uarm.read_servo_angle(SERVO_RIGHT_NUM, 0) - 20
 
             if abs(servo_1_offset) < 5.5:
                 self.manual_offset_correct_flag[0] = True
@@ -238,50 +241,55 @@ class Calibration(object):
             self.uf_print("Error - 2, Manual Calibration Servo offset not equal to EEPROM")
             self.uf_print("Error - 2, manual Servo Offset: ", self.temp_manual_offset_arr)
             self.uf_print("Error - 2, read_manual_offset: ", self.read_manual_offset())
-        self.uarm.detachAll()
+        self.uarm.detach_all_servos()
 
-    def stretch_calibration_section(self, callback):
+    def stretch_calibration_section(self, callback=None):
+        self.stretch_calibration_flag = True
         self.uf_print("3.0. Clearing Stretch Completed Flag in EEPROM.")
         self.write_completed_flag(CALIBRATION_STRETCH_FLAG, False)
         self.uf_print("3. Start Calibrate Stretch Offset")
 
         self.uf_print("3.0 Moving uArm to Correct Place")
-        self.uarm.writeServoAngle(SERVO_ROT_NUM, 45, 0)
+        self.uarm.write_servo_angle(SERVO_ROT_NUM, 45, 0)
         time.sleep(1)
-        self.uarm.writeLeftRightServoAngle(130, 20, 0)
+        self.uarm.write_left_right_servo_angle(130, 20, 0)
         time.sleep(1)
 
         initPosL = INIT_POS_L - 12
         initPosR = INIT_POS_R - 12
-        minAngle_L = self.uarm.readAnalog(SERVO_LEFT_ANALOG_PIN) - 16;
-        minAngle_R = self.uarm.readAnalog(SERVO_RIGHT_ANALOG_PIN) - 16;
+        minAngle_L = self.uarm.read_analog(SERVO_LEFT_ANALOG_PIN) - 16;
+        minAngle_R = self.uarm.read_analog(SERVO_RIGHT_ANALOG_PIN) - 16;
         print 'minAngle_L: ', minAngle_L
         print 'minAngle_R: ', minAngle_R
 
-        self.uarm.writeServoAngle(SERVO_LEFT_NUM, initPosL, 0)
-        self.uarm.writeServoAngle(SERVO_RIGHT_NUM, initPosR, 0)
+        self.uarm.write_servo_angle(SERVO_LEFT_NUM, initPosL, 0)
+        self.uarm.write_servo_angle(SERVO_RIGHT_NUM, initPosR, 0)
         time.sleep(1)
-        while self.uarm.readAnalog(SERVO_RIGHT_ANALOG_PIN) < (minAngle_R - SAMPLING_DEADZONE):
+        while self.uarm.read_analog(SERVO_RIGHT_ANALOG_PIN) < (minAngle_R - SAMPLING_DEADZONE) \
+                and self.stretch_calibration_flag:
             initPosR += 1
-            self.uarm.writeServoAngle(SERVO_RIGHT_NUM, initPosR, 0)
+            self.uarm.write_servo_angle(SERVO_RIGHT_NUM, initPosR, 0)
             print 'initPosR: ', initPosR
             time.sleep(0.05)
 
-        while self.uarm.readAnalog(SERVO_RIGHT_ANALOG_PIN) > (minAngle_R + SAMPLING_DEADZONE):
+        while self.uarm.read_analog(SERVO_RIGHT_ANALOG_PIN) > (minAngle_R + SAMPLING_DEADZONE) \
+                and self.stretch_calibration_flag:
             initPosR -= 1
-            self.uarm.writeServoAngle(SERVO_RIGHT_NUM, initPosR, 0)
+            self.uarm.write_servo_angle(SERVO_RIGHT_NUM, initPosR, 0)
             print 'initPosR: ', initPosR
             time.sleep(0.05)
 
-        while self.uarm.readAnalog(SERVO_LEFT_ANALOG_PIN) < (minAngle_L - SAMPLING_DEADZONE):
+        while self.uarm.read_analog(SERVO_LEFT_ANALOG_PIN) < (minAngle_L - SAMPLING_DEADZONE) \
+                and self.stretch_calibration_flag:
             initPosL += 1
-            self.uarm.writeServoAngle(SERVO_LEFT_NUM, initPosL, 0)
+            self.uarm.write_servo_angle(SERVO_LEFT_NUM, initPosL, 0)
             print 'initPosL: ', initPosL
             time.sleep(0.05)
 
-        while self.uarm.readAnalog(SERVO_LEFT_ANALOG_PIN) > (minAngle_L + SAMPLING_DEADZONE):
+        while self.uarm.read_analog(SERVO_LEFT_ANALOG_PIN) > (minAngle_L + SAMPLING_DEADZONE) \
+                and self.stretch_calibration_flag:
             initPosL -= 1
-            self.uarm.writeServoAngle(SERVO_LEFT_NUM, initPosL, 0)
+            self.uarm.write_servo_angle(SERVO_LEFT_NUM, initPosL, 0)
             print 'initPosL: ', initPosL
             time.sleep(0.05)
 
@@ -300,12 +308,13 @@ class Calibration(object):
             callback(stretch_offset, offset_correct_flag)
 
         self.uf_print("    3.1 Saving Stretch Offset into EEPROM")
-        self.uarm.writeEEPROM(EEPROM_DATA_TYPE_FLOAT, OFFSET_STRETCH_START_ADDRESS, offsetL)
-        self.uarm.writeEEPROM(EEPROM_DATA_TYPE_FLOAT, OFFSET_STRETCH_START_ADDRESS + 4, offsetR)
+        self.uarm.write_eeprom(EEPROM_DATA_TYPE_FLOAT, OFFSET_STRETCH_START_ADDRESS, offsetL)
+        self.uarm.write_eeprom(EEPROM_DATA_TYPE_FLOAT, OFFSET_STRETCH_START_ADDRESS + 4, offsetR)
         self.uf_print("    3.2 Mark Completed Flag in EEPROM")
         self.write_completed_flag(CALIBRATION_STRETCH_FLAG, True)
         print offsetL, offsetR
-        self.uarm.detachAll()
+        self.uarm.detach_all_servos()
+        self.stretch_calibration_flag = False
 
     def save_linear_offset(self):
         self.uf_print("    1.2 Saving Servo Offset into EEPROM")
@@ -314,9 +323,9 @@ class Calibration(object):
         for i in range(4):
             print intercept_address, self.linear_offset[i]['INTERCEPT']
             print slope_address, self.linear_offset[i]['SLOPE']
-            self.uarm.writeEEPROM(EEPROM_DATA_TYPE_FLOAT, intercept_address, self.linear_offset[i]['INTERCEPT'])
+            self.uarm.write_eeprom(EEPROM_DATA_TYPE_FLOAT, intercept_address, self.linear_offset[i]['INTERCEPT'])
             time.sleep(0.5)
-            self.uarm.writeEEPROM(EEPROM_DATA_TYPE_FLOAT, slope_address, self.linear_offset[i]['SLOPE'])
+            self.uarm.write_eeprom(EEPROM_DATA_TYPE_FLOAT, slope_address, self.linear_offset[i]['SLOPE'])
             time.sleep(0.5)
             intercept_address += 4
             slope_address += 4
@@ -325,14 +334,14 @@ class Calibration(object):
         self.uf_print("    2.1 Saving Servo Offset into EEPROM")
         address = OFFSET_START_ADDRESS
         for i in self.temp_manual_offset_arr:
-            self.uarm.writeEEPROM(EEPROM_DATA_TYPE_FLOAT, address, i)
+            self.uarm.write_eeprom(EEPROM_DATA_TYPE_FLOAT, address, i)
             address += 4
 
     def read_manual_offset(self):
         address = OFFSET_START_ADDRESS
         read_manual_offset = []
         for i in range(3):
-            read_manual_offset.append(round(self.uarm.readEEPROM(EEPROM_DATA_TYPE_FLOAT, address), 2))
+            read_manual_offset.append(round(self.uarm.read_eeprom(EEPROM_DATA_TYPE_FLOAT, address), 2))
             address += 4
         return read_manual_offset
 
@@ -342,8 +351,8 @@ class Calibration(object):
         linear_offset_data = []
         for i in range(4):
             linear_offset_template = copy.deepcopy(self.linear_offset_template)
-            linear_offset_template['INTERCEPT'] = round(self.uarm.readEEPROM(EEPROM_DATA_TYPE_FLOAT, intercept_address), 2)
-            linear_offset_template['SLOPE'] = round(self.uarm.readEEPROM(EEPROM_DATA_TYPE_FLOAT, slope_address), 2)
+            linear_offset_template['INTERCEPT'] = round(self.uarm.read_eeprom(EEPROM_DATA_TYPE_FLOAT, intercept_address), 2)
+            linear_offset_template['SLOPE'] = round(self.uarm.read_eeprom(EEPROM_DATA_TYPE_FLOAT, slope_address), 2)
             linear_offset_data.append(linear_offset_template)
             intercept_address += 4
             slope_address += 4
@@ -351,8 +360,8 @@ class Calibration(object):
 
     def read_stretch_offset(self):
         address = OFFSET_STRETCH_START_ADDRESS
-        left_offset = round(self.uarm.readEEPROM(EEPROM_DATA_TYPE_FLOAT, address), 2)
-        right_offset = round(self.uarm.readEEPROM(EEPROM_DATA_TYPE_FLOAT, address + 4), 2)
+        left_offset = round(self.uarm.read_eeprom(EEPROM_DATA_TYPE_FLOAT, address), 2)
+        right_offset = round(self.uarm.read_eeprom(EEPROM_DATA_TYPE_FLOAT, address + 4), 2)
         stretch_offset = copy.deepcopy(self.stretch_offset_template)
         stretch_offset['LEFT'] = left_offset
         stretch_offset['RIGHT'] = right_offset
@@ -366,32 +375,32 @@ class Calibration(object):
 
     def write_completed_flag(self, flag_type, flag):
         if flag_type == CALIBRATION_FLAG:
-            self.uarm.writeEEPROM(EEPROM_DATA_TYPE_BYTE, CALIBRATION_FLAG, CALIBRATION_FLAG if flag else 0)
+            self.uarm.write_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_FLAG, CONFIRM_FLAG if flag else 0)
         elif flag_type == CALIBRATION_LINEAR_FLAG:
-            self.uarm.writeEEPROM(EEPROM_DATA_TYPE_BYTE, CALIBRATION_LINEAR_FLAG, CALIBRATION_LINEAR_FLAG if flag else 0)
+            self.uarm.write_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_LINEAR_FLAG, CONFIRM_FLAG if flag else 0)
         elif flag_type == CALIBRATION_SERVO_FLAG:
-            self.uarm.writeEEPROM(EEPROM_DATA_TYPE_BYTE, CALIBRATION_SERVO_FLAG, CALIBRATION_SERVO_FLAG if flag else 0)
+            self.uarm.write_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_SERVO_FLAG, CONFIRM_FLAG if flag else 0)
         elif flag_type == CALIBRATION_STRETCH_FLAG:
-            self.uarm.writeEEPROM(EEPROM_DATA_TYPE_BYTE, CALIBRATION_STRETCH_FLAG, CALIBRATION_STRETCH_FLAG if flag else 0)
+            self.uarm.write_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_STRETCH_FLAG, CONFIRM_FLAG if flag else 0)
 
     def read_completed_flag(self, flag_type):
         if flag_type == CALIBRATION_FLAG:
-            if self.uarm.readEEPROM(EEPROM_DATA_TYPE_BYTE, CALIBRATION_FLAG) == CALIBRATION_FLAG:
+            if self.uarm.read_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_FLAG) == CONFIRM_FLAG:
                 return True
             else:
                 return False
         elif flag_type == CALIBRATION_LINEAR_FLAG:
-            if self.uarm.readEEPROM(EEPROM_DATA_TYPE_BYTE, CALIBRATION_LINEAR_FLAG) == CALIBRATION_LINEAR_FLAG:
+            if self.uarm.read_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_LINEAR_FLAG) == CONFIRM_FLAG:
                 return True
             else:
                 return False
         elif flag_type == CALIBRATION_SERVO_FLAG:
-            if self.uarm.readEEPROM(EEPROM_DATA_TYPE_BYTE, CALIBRATION_SERVO_FLAG) == CALIBRATION_SERVO_FLAG:
+            if self.uarm.read_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_SERVO_FLAG) == CONFIRM_FLAG:
                 return True
             else:
                 return False
         elif flag_type == CALIBRATION_STRETCH_FLAG:
-            if self.uarm.readEEPROM(EEPROM_DATA_TYPE_BYTE, CALIBRATION_STRETCH_FLAG) == CALIBRATION_STRETCH_FLAG:
+            if self.uarm.read_eeprom(EEPROM_DATA_TYPE_BYTE, CALIBRATION_STRETCH_FLAG) == CONFIRM_FLAG:
                 return True
             else:
                 return False
