@@ -5,13 +5,13 @@ from serial.tools import list_ports
 
 # version
 MAJOR_VERSION = 1
-MINOR_VERSION = 1
-BUGFIX_VERSION = 8
+MINOR_VERSION = 2
+BUGFIX_VERSION = 7
 VERSION = str(MAJOR_VERSION) + "." + str(MINOR_VERSION) + "." + str(BUGFIX_VERSION)
 
 # Firmata
 
-REPORT_VERSION = 0x79
+REPORT_FIRMATA_VERSION = 0x79
 
 # parameters
 SERVO_ROT_NUM           = 0
@@ -46,7 +46,10 @@ WRITE_LEFT_RIGHT_ANGLE      = 0X1F
 GRIPPER_STATUS              = 0X20
 READ_SERIAL_NUMBER          = 0x21
 WRITE_SERIAL_NUMBER         = 0x22
+REPORT_LIBRARY_VERSION      = 0x23
+BUZZER_ALERT                = 0x24
 
+CONFIRM_FLAG = 0x80
 CALIBRATION_FLAG                    = 10
 CALIBRATION_LINEAR_FLAG             = 11
 CALIBRATION_SERVO_FLAG              = 12
@@ -57,14 +60,17 @@ LINEAR_SLOPE_START_ADDRESS          = 50
 OFFSET_START_ADDRESS                = 30
 OFFSET_STRETCH_START_ADDRESS        = 20
 
-SERIAL_NUMBER_ADDRESS = 1024
+SERIAL_NUMBER_ADDRESS = 100
 
 EEPROM_DATA_TYPE_BYTE               = 1
+EEPROM_DATA_TYPE_INTEGER            = 2
 EEPROM_DATA_TYPE_FLOAT              = 4
 
 BUTTON_D7 = 7
 BUTTON_D4 = 4
 BUTTON_D2 = 2
+PUMP_PIN = 6
+VALVE_PIN = 5
 
 PULL_UP = 1
 INPUT = 0
@@ -91,14 +97,23 @@ def get_uarm():
 class uArm(object):
 
     firmware_major_version  = 0
+    firmware_minor_version = 0
+    firmware_bugfix = 0
 
-    def __init__(self,port):
+    frimata_major_version = 0
+    frimata_minor_version = 0
+
+    def __init__(self,port,debug_mode=False):
         self.port = port
         self.sp = serial.Serial(port,baudrate=57600)
-        self.setFirmwareVersion()
+        self.debug_mode = debug_mode
+        self.set_firmata_version()
+        print self.get_firmata_version()
+        self.set_frimware_version()
+        print self.get_firmware_version()
 
     def isConnected(self):
-        if self.sp.isOpen() == False:
+        if not self.sp.isOpen():
             return False
         else:
             return True
@@ -109,32 +124,42 @@ class uArm(object):
     def reconnect(self):
         if not self.isConnected():
             self.sp.open()
-            self.setFirmwareVersion()
+            self.set_firmata_version()
+            print self.get_firmata_version()
+            self.set_frimware_version()
+            print self.get_firmware_version()
 
-    def setFirmwareVersion(self):
+    def get_firmware_version(self):
+        return str(self.firmware_major_version) + "." + str(self.firmware_minor_version) + "." + str(self.firmware_bugfix)
+
+    def get_firmata_version(self):
+        return str(self.frimata_major_version) + "." + str(self.frimata_minor_version)
+
+    def set_firmata_version(self):
         try:
-            msg = bytearray([START_SYSEX, REPORT_VERSION, END_SYSEX])
-            self.sp.write(msg)
+            # self.writeSerialMsg(msg)
             while self.sp.readable():
-                readData = ord(self.sp.read(1))
-                received_data = []
+                read_byte = self.sp.read(1)
+                # print binascii.hexlify(read_byte)
+                readData = ord(read_byte)
+                if readData == END_SYSEX:
+                    break
                 if readData == START_SYSEX:
                     readData = ord(self.sp.read(1))
-                    if readData == REPORT_VERSION:
-                        self.firmware_major_version = ord(self.sp.read(1))
-                        self.firmware_minor_version = ord(self.sp.read(1))
-                        break
+                    if readData == REPORT_FIRMATA_VERSION:
+                        self.frimata_major_version = ord(self.sp.read(1))
+                        self.frimata_minor_version = ord(self.sp.read(1))
         except Exception as e:
-            print "Serial Exception: ",e.strerror
+            print "Serial Exception: ",e.message
 
-    def readServoAngle(self,servo_add,data_offset):
+    def read_servo_angle(self, servo_add, data_offset):
         msg = bytearray([START_SYSEX, UARM_CODE, READ_ANGLE])
         msg.extend(getValueAsOne7bitBytes(servo_add))
         msg.extend(getValueAsOne7bitBytes(data_offset))
         msg.append(END_SYSEX)
         # print binascii.hexlify(bytearray(msg))
 
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
         while self.sp.readable():
             readData = ord(self.sp.read(1))
             received_data = []
@@ -148,37 +173,43 @@ class uArm(object):
                     if received_data[0] == servo_add:
                         return received_data[2]*128+received_data[1]+received_data[3]/100.00
 
-    def readEEPROM(self,data_type, eeprom_add):
-        msg = bytearray([START_SYSEX, UARM_CODE, READ_EEPROM])
-        msg.append(data_type)
-        msg.extend(getValueAsTwo7bitBytes(eeprom_add))
-        msg.append(END_SYSEX)
-        # print binascii.hexlify(bytearray(msg))
+    def read_eeprom(self, data_type, eeprom_add):
+        if self.isConnected():
+            msg = bytearray([START_SYSEX, UARM_CODE, READ_EEPROM])
+            msg.append(data_type)
+            msg.extend(getValueAsTwo7bitBytes(eeprom_add))
+            msg.append(END_SYSEX)
 
-        self.sp.write(msg)
-        while self.sp.readable():
-            readData = ord(self.sp.read(1))
-            received_data = []
-            if (readData == START_SYSEX):
+            self.write_serial_msg(msg)
+            while self.sp.readable():
                 readData = ord(self.sp.read(1))
-                if (readData == UARM_CODE):
+                received_data = []
+                if (readData == START_SYSEX):
                     readData = ord(self.sp.read(1))
-                    if (readData == READ_EEPROM):
+                    if (readData == UARM_CODE):
                         readData = ord(self.sp.read(1))
-                        while readData != END_SYSEX:
-                            received_data.append(readData)
+                        if (readData == READ_EEPROM):
                             readData = ord(self.sp.read(1))
-                        res_eeprom_add = received_data[1]*128 + received_data[0]
-                        # print res_eeprom_add
-                        if res_eeprom_add == eeprom_add:
-                            if data_type == EEPROM_DATA_TYPE_BYTE:
-                                return (received_data[3]<<7)+received_data[2]
-                            if data_type == EEPROM_DATA_TYPE_FLOAT:
-                                val = (received_data[4]<<7)+received_data[3]+float(received_data[5])/100.00
-                                val =  val if received_data[2] == 0 else -val
-                                return val
+                            while readData != END_SYSEX:
+                                received_data.append(readData)
+                                readData = ord(self.sp.read(1))
+                            res_eeprom_add = received_data[1]*128 + received_data[0]
+                            # print res_eeprom_add
+                            if res_eeprom_add == eeprom_add:
+                                if data_type == EEPROM_DATA_TYPE_BYTE:
+                                    return (received_data[3]<<7)+received_data[2]
+                                if data_type == EEPROM_DATA_TYPE_FLOAT:
+                                    val = (received_data[4]<<7)+received_data[3]+float(received_data[5])/100.00
+                                    val =  val if received_data[2] == 0 else -val
+                                    return val
+                                if data_type == EEPROM_DATA_TYPE_INTEGER:
+                                    val = (received_data[4]<<7)+received_data[3]
+                                    val =  val if received_data[2] == 0 else -val
+                                    return val
+        else:
+            print "uArm is not connected"
 
-    def readDigital(self,pin_number, pin_mode):
+    def read_digital(self, pin_number, pin_mode):
         pin_number = int (pin_number)#(sys.argv[2])
         pin_mode = (int(pin_mode) == 1 )and 1 or 0  # 1means pullup 0 means input
         msg = bytearray([START_SYSEX, UARM_CODE, READ_DIGITAL])
@@ -186,7 +217,7 @@ class uArm(object):
         msg.extend(getValueAsOne7bitBytes(pin_mode))
         msg.append(END_SYSEX)
 
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
         while self.sp.readable():
             readData = ord(self.sp.read(1))
             received_data = []
@@ -200,13 +231,13 @@ class uArm(object):
                     if received_data[0] == pin_number:
                         return received_data[1]
 
-    def readAnalog(self,pin_num):
+    def read_analog(self, pin_num):
         msg = bytearray([START_SYSEX, UARM_CODE, READ_ANALOG])
         msg.extend(getValueAsOne7bitBytes(pin_num))
         msg.append(END_SYSEX)
         # print binascii.hexlify(bytearray(msg))
 
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
         while self.sp.readable():
             readData = ord(self.sp.read(1))
             received_data = []
@@ -221,11 +252,11 @@ class uArm(object):
                         return received_data[2]*128+received_data[1]
 
 
-    def readCoords(self,pin_num):
+    def read_coords(self, pin_num):
         msg = bytearray([START_SYSEX, UARM_CODE, READ_COORDS])
         msg.extend(getValueAsOne7bitBytes(pin_num))
         msg.append(END_SYSEX)
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
         while self.sp.readable():
             readData = ord(self.sp.read(1))
             received_data = []
@@ -246,25 +277,27 @@ class uArm(object):
                         coords.append(coords_sign*coords_val)
                     return coords
 
-    def writeEEPROM(self,data_type, eeprom_add, eeprom_val):
+    def write_eeprom(self, data_type, eeprom_add, eeprom_val):
         msg = bytearray([START_SYSEX, UARM_CODE, WRITE_EEPROM, data_type])
         msg.extend(getValueAsTwo7bitBytes(eeprom_add))
         if data_type == EEPROM_DATA_TYPE_BYTE:
             msg.extend(getValueAsTwo7bitBytes(eeprom_val))
         if data_type == EEPROM_DATA_TYPE_FLOAT:
-            msg.extend(getValueAsFour7bitBytes(eeprom_val))
+            msg.extend(getFloatAsFour7bitBytes(eeprom_val))
             # print getValueAsFour7bitBytes(eeprom_val)[0]
+        if data_type == EEPROM_DATA_TYPE_INTEGER:
+            msg.extend(getIntegerAsThree7bitBytes(eeprom_val))
         msg.append(END_SYSEX)
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
 
-    def writeAnalog(self,pin_number,pin_value):
+    def write_analog(self, pin_number, pin_value):
         msg = bytearray([START_SYSEX, UARM_CODE, WRITE_ANALOG])
         msg.extend(getValueAsOne7bitBytes(pin_number))
         msg.extend(getValueAsTwo7bitBytes(pin_value))
         msg.append(END_SYSEX)
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
 
-    def writeDigital(self,pin_number,pin_mode):
+    def write_digtial(self, pin_number, pin_mode):
         pin_number = int (pin_number)
         pin_mode = ((int (pin_mode)) ==1) and 1 or 0
 
@@ -272,96 +305,96 @@ class uArm(object):
         msg.extend(getValueAsOne7bitBytes(pin_number))
         msg.extend(getValueAsOne7bitBytes(pin_mode))
         msg.append(END_SYSEX)
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
 
-    def detachAll(self):
+    def detach_all_servos(self):
         msg = bytearray([START_SYSEX, UARM_CODE, DETACH_SERVO, END_SYSEX])
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
 
-    def writeServoAngle(self,servo_number,servo_angle,writeWithoffset):
+    def write_servo_angle(self, servo_number, servo_angle, writeWithoffset):
         servo_number = int(servo_number)
         servo_angle = float(servo_angle)
         writeWithoffset = int(writeWithoffset)
         msg = bytearray([START_SYSEX, UARM_CODE, WRITE_ANGLE])
         msg.extend(getValueAsOne7bitBytes(servo_number))
-        msg.extend(getValueAsThree7bitBytes(servo_angle))
+        msg.extend(getFloatAsThree7bitBytes(servo_angle))
         msg.extend(getValueAsOne7bitBytes(writeWithoffset))
         msg.append(END_SYSEX)
         # print binascii.hexlify(bytearray(msg))
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
 
-    def writeLeftRightServoAngle(self, servo_left_angle, servo_right_angle, writeWithoffset):
+    def write_left_right_servo_angle(self, servo_left_angle, servo_right_angle, writeWithoffset):
         writeWithoffset = int(writeWithoffset)
         servo_left_angle = float(servo_left_angle)
         servo_right_angle = float(servo_right_angle)
         msg = bytearray([START_SYSEX, UARM_CODE, WRITE_LEFT_RIGHT_ANGLE])
-        msg.extend(getValueAsThree7bitBytes(servo_left_angle))
-        msg.extend(getValueAsThree7bitBytes(servo_right_angle))
+        msg.extend(getFloatAsThree7bitBytes(servo_left_angle))
+        msg.extend(getFloatAsThree7bitBytes(servo_right_angle))
         msg.extend(getValueAsOne7bitBytes(writeWithoffset))
         msg.append(END_SYSEX)
         # print binascii.hexlify(bytearray(msg))
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
 
     def move(self,x,y,z):
         x,y,z = float(x), float(y), float(z)
-        self.moveToOpts(x,y,z,0,1,0,0,0)
+        self.move_to_options(x, y, z, 0, 1, 0, 0, 0)
 
-    def moveTo(self,x,y,z):
+    def move_to(self, x, y, z):
         x,y,z = float(x), float(y), float(z)
-        self.moveToOpts(x,y,z,0,0,0,0,0)
+        self.move_to_options(x, y, z, 0, 0, 0, 0, 0)
 
-    def pumpStatus(self, val):
+    def pump_control(self, val):
         pump_status = 1 if val else 0
         msg = bytearray([START_SYSEX, UARM_CODE, PUMP_STATUS])
         msg.extend(getValueAsOne7bitBytes(pump_status))
         msg.append(END_SYSEX)
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
 
-    def gripperStatus(self, val):
+    def gripper_status(self, val):
         gripper_status = 1 if val else 0
         msg = bytearray([START_SYSEX, UARM_CODE, GRIPPER_STATUS])
         msg.extend(getValueAsOne7bitBytes(gripper_status))
         msg.append(END_SYSEX)
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
 
-    def moveToOpts(self,x,y,z,hand_angle,relative_flags,time_spend,path_type,ease_type):
+    def move_to_options(self, x, y, z, hand_angle, relative_flags, time_spend, path_type, ease_type):
         msg = bytearray([START_SYSEX, UARM_CODE, WRITE_COORDS])
-        msg.extend(getValueAsFour7bitBytes(x))
-        msg.extend(getValueAsFour7bitBytes(y))
-        msg.extend(getValueAsFour7bitBytes(z))
-        msg.extend(getValueAsThree7bitBytes(hand_angle))
+        msg.extend(getFloatAsFour7bitBytes(x))
+        msg.extend(getFloatAsFour7bitBytes(y))
+        msg.extend(getFloatAsFour7bitBytes(z))
+        msg.extend(getFloatAsThree7bitBytes(hand_angle))
         msg.extend(getValueAsOne7bitBytes(relative_flags))
-        msg.extend(getValueAsThree7bitBytes(time_spend))
+        msg.extend(getFloatAsThree7bitBytes(time_spend))
         msg.extend(getValueAsOne7bitBytes(path_type))
         msg.extend(getValueAsOne7bitBytes(ease_type))
         # msg.append(1 if enable_hand else 0)
         msg.append(END_SYSEX)
         time.sleep(0.01)
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
 
-    def moveToSimple(self,x,y,z,hand_angle,relative_flags,time_spend):
-        self.moveToOpts(x,y,z,hand_angle,relative_flags,time_spend,0,0, True)
+    def move_to_simple(self, x, y, z, hand_angle, relative_flags, time_spend):
+        self.move_to_options(x, y, z, hand_angle, relative_flags, time_spend, 0, 0)
 
-    def writeStretch(self,length,height):
+    def write_stretch(self, length, height):
         length = float(length)
         height = float(height)
-
         msg = bytearray([START_SYSEX, UARM_CODE, WRITE_STRETCH])
-        msg.extend(getValueAsFour7bitBytes(length))
-        msg.extend(getValueAsFour7bitBytes(height))
+        msg.extend(getFloatAsFour7bitBytes(length))
+        msg.extend(getFloatAsFour7bitBytes(height))
         msg.append(END_SYSEX)
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
 
-    def writeSerialNumber(self, serial_number):
+    def write_serial_number(self, serial_number):
         msg = bytearray([START_SYSEX, UARM_CODE, WRITE_SERIAL_NUMBER])
         for c in serial_number:
             msg.append(ord(c))
         msg.append(END_SYSEX)
-        self.sp.write(msg)
+        # print binascii.hexlify(msg)
+        self.write_serial_msg(msg)
 
-    def readSerialNumber(self):
+    def read_serial_number(self):
         msg = bytearray([START_SYSEX, UARM_CODE, READ_SERIAL_NUMBER, END_SYSEX])
-        self.sp.write(msg)
+        self.write_serial_msg(msg)
         while self.sp.readable():
             readData = ord(self.sp.read(1))
             received_data = []
@@ -375,11 +408,44 @@ class uArm(object):
                             received_data.append(readData)
                             readData = ord(self.sp.read(1))
                         sn_array = []
-                        # print received_data
                         for r in received_data:
+                            # print r
                             sn_array.append(chr(r))
                         return ''.join(sn_array)
 
+    def alert(self, times, run_time, stop_time):
+        if self.isConnected():
+            msg = bytearray([START_SYSEX, UARM_CODE, BUZZER_ALERT, times, run_time, stop_time,END_SYSEX ])
+            self.write_serial_msg(msg)
+
+    def set_frimware_version(self):
+        if self.isConnected():
+            msg = bytearray([START_SYSEX, UARM_CODE, REPORT_LIBRARY_VERSION, END_SYSEX])
+            self.write_serial_msg(msg)
+            while self.sp.readable():
+                read_byte = self.sp.read(1)
+                readData = ord(read_byte)
+                received_data = []
+                if (readData == START_SYSEX):
+                    readData = ord(self.sp.read(1))
+                    if (readData == UARM_CODE):
+                        readData = ord(self.sp.read(1))
+                        if (readData == REPORT_LIBRARY_VERSION):
+                            readData = ord(self.sp.read(1))
+                            while readData != END_SYSEX:
+                                received_data.append(readData)
+                                readData = ord(self.sp.read(1))
+                            self.firmware_major_version = received_data[0]
+                            self.firmware_minor_version = received_data[1]
+                            self.firmware_bugfix = received_data[2]
+                            break
+        else:
+            print "uArm is not Connected"
+
+    def write_serial_msg(self, msg):
+        if self.debug_mode:
+            print binascii.hexlify(msg)
+        self.sp.write(msg)
 
 class ConnectError(RuntimeError):
    def __init__(self, arg):
@@ -392,13 +458,16 @@ def getValueAsTwo7bitBytes(val):
     int_val = int(val)
     return bytearray([abs(int_val) % 128, abs(int_val) >> 7])
 
-def getValueAsThree7bitBytes(val):
+def getFloatAsThree7bitBytes(val):
     int_val = int(val)
     decimal_val = int(round((val - int_val) * 100))
     # print decimal_val
     return bytearray([abs(int_val) % 128, abs(int_val) >> 7, abs(decimal_val)])
 
-def getValueAsFour7bitBytes(val):
+def getIntegerAsThree7bitBytes(val):
+    return bytearray([0 if val > 0 else 1, abs(val) % 128, abs(val) >> 7])
+
+def getFloatAsFour7bitBytes(val):
     int_val = int(val)
     decimal_val = int(round((val - int_val) * 100))
     # print "decimal_val: ", decimal_val
